﻿#ifndef __DING_RTC_ENGINE_INTERFACE_H__
#define __DING_RTC_ENGINE_INTERFACE_H__

#include "engine_types.h"
#include "engine_utils.h"
#include "engine_device_manager.h"
#include "engine_audio_mixing_manager.h"
#include "engine_wb_interface.h"

namespace ding {
namespace rtc {


/**
 * \defgroup CPP_DingRtcEngine 全部接口
 * DingRtc 接口
 */

/** \defgroup CPP_DingRtcEngineBase 基础接口
 *  \ingroup CPP_DingRtcEngine
 *  DingRtc 基础接口
 */

 /** \defgroup CPP_DingRtcEngineChannel 频道相关接口
 *  \ingroup CPP_DingRtcEngine
 *  DingRtc 频道相关接口
 */

/** \defgroup CPP_DingRtcEnginePub 发布相关接口
 *  \ingroup CPP_DingRtcEngine
 *  DingRtc 发布相关接口
 */

 /** \defgroup CPP_DingRtcEngineSub 订阅相关接口
 *  \ingroup CPP_DingRtcEngine
 *  DingRtc 订阅相关接口
 */

/** \defgroup CPP_DingRtcEngineVideo 视频相关接口
 *  \ingroup CPP_DingRtcEngine
 *  DingRtc 视频相关接口
 */

 /** \defgroup CPP_DingRtcEngineScreenShare 共享视频相关接口
 *  \ingroup CPP_DingRtcEngine
 *  DingRtc 视频相关接口
 */

 /** \defgroup CPP_DingRtcEngineAudio 音频相关接口
 *  \ingroup CPP_DingRtcEngine
 *  DingRtc 音频相关接口
 */

/** \defgroup CPP_DingRtcEngineMediaEngine 媒体引擎相关接口
 *  \ingroup CPP_DingRtcEngine
 *  DingRtc 媒体引擎相关接口
 */

 /** \defgroup CPP_DingRtcEngineLiveBypass 直播旁路相关接口
 *  \ingroup CPP_DingRtcEngine
 *  DingRtc 直播旁路相关接口
 */

/** \defgroup CPP_DingRtcEngineChannelRelay 跨频道连麦相关接口
 *  \ingroup CPP_DingRtcEngine
 *  DingRtc 跨频道连麦相关接口
 */

 /** \defgroup CPP_DingRtcEnginePreview 预览相关接口
 *  \ingroup CPP_DingRtcEngine
 *  DingRtc 预览相关接口
 */

/** \defgroup CPP_DingRtcEngineUserInfo 远端用户查询接口
 *  \ingroup CPP_DingRtcEngine
 *  DingRtc 远端用户查询接口
 */

 /** \defgroup CPP_DingRtcEngineOthers 其他接口
 *  \ingroup CPP_DingRtcEngine
 *  DingRtc 其他接口
 */

/**
* \defgroup CPP_DingRtcDef 关键类型定义
* DingRtc 关键类型定义
*/

/**
* \defgroup CPP_DingRtcCallback 回调接口
* DingRtc 关键类型定义
*/

/**
 * @defgroup CPP_DingRtcCallbackVideoFrameObserver 视频数据回调及监听
 * @ingroup CPP_DingRtcCallback
 * DingRtc 视频数据回调及监听
 * @{
 */

/**
 * @brief Video observer使用
 * 
 */
class RtcEngineVideoFrameObserver {
public:
    virtual ~RtcEngineVideoFrameObserver() {}

    /**
     * @ingroup CPP_DingRtcCallbackVideoFrameObserver
     * @since 3.0
     * @brief 视频格式采集偏好。
    */
    virtual RtcEngineVideoPixelFormat GetVideoFormatPreference() = 0;

    /**
     * @ingroup CPP_DingRtcCallbackVideoFrameObserver
     * @since 3.0
     * @brief 本地采集视频数据。
    */
    virtual bool OnCaptureVideoFrame(ding::rtc::RtcEngineVideoFrame &frame) = 0;
    /**
     * @ingroup CPP_DingRtcCallbackVideoFrameObserver
     * @since 3.1
     * @brief 远端解码后视频数据。
    */
    virtual bool OnRemoteVideoFrame(ding::rtc::String uid, ding::rtc::RtcEngineVideoTrack track, ding::rtc::RtcEngineVideoFrame &frame) = 0;
    /**
     * @ingroup CPP_DingRtcCallbackVideoFrameObserver
     * @since 3.1
     * @brief 本地编码前视频数据。
    */
    virtual bool OnPreEncodeVideoFrame(ding::rtc::RtcEngineVideoTrack track, ding::rtc::RtcEngineVideoFrame &frame) = 0;
};

class DING_RTC_API RtcEngineVideoStreamSink {
public:
    RtcEngineVideoStreamSink();
    virtual ~RtcEngineVideoStreamSink();

    // app need implement this callback
    virtual void OnVideoFrame(ding::rtc::RtcEngineVideoFrame &frame) = 0;

    // sdk internally used
    void *GetSinkImpl();
private:
    void *impl_ = 0;
};
/**
 * @}
 */

/**
 * @defgroup CPP_DingRtcCallbackAudioFrameObserver 音频数据回调及监听
 * @ingroup CPP_DingRtcCallback
 * DingRtc 音频数据回调及监听。
 * @{
 */

class RtcEngineAudioFrameObserver {
public:
    virtual ~RtcEngineAudioFrameObserver() {}

    /**
     * @brief 本地订阅音频数据回调。
     * @details 远端所有用户混音后待播放的音频数据。
     * @param frame 音频数据，详见{@link RtcEngineAudioFrame}。
     * @since 3.0
    */
    virtual void OnPlaybackAudioFrame(ding::rtc::RtcEngineAudioFrame &frame) = 0;

    /**
     * @brief 本地采集音频数据回调。
     * @param frame 音频数据，详见{@link RtcEngineAudioFrame}。
     * @since 3.0
     */
    virtual void OnCapturedAudioFrame(ding::rtc::RtcEngineAudioFrame &frame) = 0;

    /**
     * @brief 经过3A处理后的数据回调。
     * @param frame 音频数据，详见{@link RtcEngineAudioFrame}。
     * @since 3.0
    */
    virtual void OnProcessCapturedAudioFrame(ding::rtc::RtcEngineAudioFrame &frame) = 0;

    /**
     * @brief 本地推流音频数据回调。
     * @param frame 音频数据，详见{@link RtcEngineAudioFrame}。
     * @since 3.0
     */
    virtual void OnPublishAudioFrame(ding::rtc::RtcEngineAudioFrame &frame) = 0;
};
/**
 * @}
 */


/**
 * @defgroup CPP_DingRtcCallbackEventListener 回调及监听
 * @ingroup CPP_DingRtcCallback
 * DingRtc 回调及监听
 * @{
 */

/**
 * @brief SDK事件回调基础类。
 */
class RtcEngineEventListener {
public:
    virtual ~RtcEngineEventListener() {}
    
    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 加入频道结果。
     * @details 当应用调用 {@link RtcEngine::JoinChannel} 方法时，该回调表示成功/失败加入频道，并且返回频道加入的相关信息以及加入频道耗时。
     * @param result 加入频道结果，成功返回0，失败返回错误码。
     * @param channel 频道id。
     * @param userId  用户ID。
     * @param elapsed 加入频道耗时。
     */
    virtual void OnJoinChannelResult(int result, const char *channel, const char *userId, int elapsed) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 离开频道结果。
     * @details 应用调用 {@link RtcEngine::LeaveChannel} 方法时，该回调表示成功/失败离开频道，回调将会返回离会的result和该频道的基本信息,如果 {@link RtcEngine::LeaveChannel} 后直接 {@link RtcEngine::Destroy} SDK，将不会收到此回调。
     * @param result 离开频道结果，成功返回0，失败返回错误码。
     * @param stats 本次频道内会话的数据统计汇总。
     */
    virtual void OnLeaveChannelResult(int result, ding::rtc::RtcEngineStats stats) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 频道剩余时长回调。
     * @param remainingTimeInSec 频道剩余时长，单位秒。
     */
    virtual void OnChannelRemainingTimeNotify(int remainingTimeInSec) {}
    
    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 远端用户加入频道回调。
     * @param uid 用户ID 从App server分配的唯一标示符。
     * @param elapsed 用户加入频道时的耗时。
     */
    virtual void OnRemoteUserOnLineNotify(const char *uid, int elapsed) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 远端用户离开频道回调。
     * @param uid 用户ID 从App server分配的唯一标示符。
     * @param reason 用户离线的原因，详见 {@link RtcEngineUserOfflineReason}。
     */
    virtual void OnRemoteUserOffLineNotify(const char *uid, ding::rtc::RtcEngineUserOfflineReason reason) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 被服务器踢出频道的消息。
     * @param code onBye类型，详见 {@link RtcEngineOnByeType}。
     */
    virtual void OnBye(RtcEngineOnByeType code) {}
    
    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 网络连接状态改变的回调。
     * @param status 当前网络链接状态。
     * @param reason 网络链接状态改变原因。
     */
    virtual void OnConnectionStatusChanged(ding::rtc::RtcEngineConnectionStatus status, ding::rtc::RtcEngineConnectionStatusChangeReason reason) {}
    
    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 网络质量变化时发出的消息。
     * @param uid  网络质量发生变化的用户uid。
     * @param upQuality 上行网络质量，详见 {@link RtcEngineNetworkQuality}。
     * @param downQuality 下行网络质量，详见 {@link RtcEngineNetworkQuality}。
     * @note 当网络质量发生变化时触发，uid为空时代表用户自己的网络质量发生变化。
     */
    virtual void OnNetworkQualityChanged(const char *uid,
                                         ding::rtc::RtcEngineNetworkQuality upQuality,
                                         ding::rtc::RtcEngineNetworkQuality downQuality) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 埋点log。
     * @param log 埋点统计信息。
     * @note 用处不大，将来版本移除。
     */
    virtual void OnStatisticsLog(String const& log) {};
    
    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 如果engine出现warning，通过这个消息通知app。
     * @param warn 警告类型。
     * @param msg 警告信息。
     */
    virtual void OnOccurWarning(int warn, const char *msg) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 如果engine出现error，通过这个消息通知app。
     * @param error  错误类型，参考 {@link RtcEngineErrorCode}。
     * @param msg 错误描述。
     */
    virtual void OnOccurError(int error, const char* msg) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 埋点log。
     * @param level 日志等级。
     * @param log 埋点统计信息。
     * @note 用处不大，将来版本移除。
     */
    virtual void OnStatisticsLevelLog(int level, const String& log) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 质量报告，相对OnLocalAudioStats，OnRemoteVideoStats更加简练，适合在UI上展示。
     * @param quality 采集到的各项指标数据。
     */
    virtual void OnQualityReport(const ding::rtc::QualityReport& quality) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 本地音频统计信息。
     * @param localAudioStats 本地音频统计信息。
     * @note SDK周期触发一次此统计信息回调。
     */
    virtual void OnLocalAudioStats(const ding::rtc::LocalAudioStats& localAudioStats) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 本地视频统计信息。
     * @param localVideoStats 本地视频统计信息。
     * @note SDK周期触发一次此统计信息回调。
     */
    virtual void OnLocalVideoStats(const ding::rtc::LocalVideoStats& localVideoStats) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 远端音频统计信息。
     * @param remoteAudioStats 远端音频统计信息。
     * @note SDK周期触发一次此统计信息回调。
     */
    virtual void OnRemoteAudioStats(const ding::rtc::RemoteAudioStats& remoteAudioStats) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 远端视频统计信息。
     * @param remoteVideoStats 远端视频统计信息。
     * @note SDK周期触发一次此统计信息回调。
     */
    virtual void OnRemoteVideoStats(const ding::rtc::RemoteVideoStats& remoteVideoStats) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 当前会话统计信息回调。
     * @param stats 会话统计信息。
     * @note SDK周期触发一次此统计信息回调。
     */
    virtual void OnStats(const ding::rtc::RtcEngineStats& stats) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 音频推流变更回调。
     * @param oldState 之前的推流状态。
     * @param newState 当前的推流状态。
     * @param elapseSinceLastState 状态变更时间间隔。
     * @param channel 当前频道id。
     */
    virtual void OnAudioPublishStateChanged(ding::rtc::RtcEnginePublishState oldState,
                                            ding::rtc::RtcEnginePublishState newState,
                                            int elapseSinceLastState,
                                            const char* channel){};

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 视频推流变更回调。
     * @param oldState 之前的推流状态。
     * @param newState 当前的推流状态。
     * @param elapseSinceLastState 状态变更时间间隔。
     * @param channel 当前频道id。
     */
    virtual void OnVideoPublishStateChanged(ding::rtc::RtcEnginePublishState oldState,
                                            ding::rtc::RtcEnginePublishState newState,
                                            int elapseSinceLastState,
                                            const char* channel){};

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 屏幕分享推流变更回调。
     * @param oldState 之前的推流状态。
     * @param newState 当前的推流状态。
     * @param elapseSinceLastState 状态变更时间间隔。
     * @param channel 当前频道id。
     */
    virtual void OnScreenSharePublishStateChanged(ding::rtc::RtcEnginePublishState oldState,
                                                  ding::rtc::RtcEnginePublishState newState,
                                                  int elapseSinceLastState,
                                                  const char* channel){};

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 远端用户的音视频流发生变化回调。
     * @details 该回调在以下场景会被触发：
     * - 当远端用户从未推流变更为推流（包括音频和视频）。
     * - 当远端用户从已推流变更为未推流（包括音频和视频）。
     * @param uid userId，从App server分配的唯一标示符。
     * @param audioTrack 音频流类型，详见 {@link RtcEngineAudioTrack}。
     * @param videoTrack 视频流类型，详见 {@link RtcEngineVideoTrack}。
     */
    virtual void OnRemoteTrackAvailableNotify(const char *uid,
                                              ding::rtc::RtcEngineAudioTrack audioTrack,
                                              ding::rtc::RtcEngineVideoTrack videoTrack) {};
    
    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 远端用户静音/取消静音回调。
     * @param uid 远端用户ID。
     * @param isMute 
     * - 该用户是否静音。
     * - true：静音。
     * - false：取消静音。
     */
    virtual void OnUserAudioMuted(const char* uid, bool isMute) {}
    
    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 远端用户暂停/恢复视频流发送的通知。
     * @param uid 远端用户ID。
     * @param isMute
     *        - true：该用户暂停视频流的发送。
     *        - false：该用户恢复视频流的发送。
     * @param videoTrack 
     *        - 屏幕流或者相机流，参考 {@link RtcEngineVideoTrack}。
     */
    virtual void OnUserVideoMuted(const char* uid, bool isMute, ding::rtc::RtcEngineVideoTrack videoTrack) {};

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.5
     * @brief 远端用户关闭相机流采集发送通知。
     * @param uid 远端用户ID。
     * @param enabled
     *        - true：打开相机流采集。
     *        - false：关闭相机流采集。
     */
    virtual void OnUserVideoEnabled(const char* uid, bool enabled) {};

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.5
     * @brief 远端用户应用退到后台
     * @param uid 远端用户ID
     */
    virtual void OnUserWillResignActive(const char* uid) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.5
     * @brief 远端用户应用返回前台
     * @param uid 远端用户ID
     */
    virtual void OnUserWillBecomeActive(const char* uid) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 远端用户的第一帧视频帧显示时触发这个消息。
     * @param uid user id。
     * @param videoTrack 屏幕流或者相机流，参考 {@link RtcEngineVideoTrack}。
     * @param width 视频宽度。
     * @param height 视频高度。
     * @param elapsed 本地用户加入频道直至该回调触发的延迟总耗时（毫秒）。
     * @note 该接口用于远端用户的第一帧视频帧显示时的回调。
     */
    virtual void OnFirstRemoteVideoFrameDrawn(const char *uid, ding::rtc::RtcEngineVideoTrack videoTrack, int width, int height, int elapsed) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 预览开始显示第一帧视频帧时触发这个消息。
     * @param width 本地预览视频宽度。
     * @param height 本地预览视频高度。
     * @param elapsed 从本地用户加入频道直至该回调触发的延迟总耗时（毫秒）。
     * @note 该接口用于预览开始显示第一帧视频帧时的回调。
     */
    virtual void OnFirstLocalVideoFrameDrawn(int width, int height, int elapsed) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 视频首包发送回调。
     * @param videoTrack 发送视频track。
     * @param timeCost 耗时（毫秒）。
     * @note 该接口用于视频首包发送的回调。
     */
    virtual void OnFirstVideoPacketSend(ding::rtc::RtcEngineVideoTrack videoTrack, int timeCost) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 视频首包接收回调。
     * @param uid User ID，从App server分配的唯一标示符。
     * @param videoTrack 接收视频track。
     * @param timeCost 耗时（毫秒）。
     * @note 该接口用于视频首包接收的回调。
     */
    virtual void OnFirstVideoPacketReceived(const char* uid,
                                            ding::rtc::RtcEngineVideoTrack videoTrack,
                                            int timeCost) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 收到远端用户视频首帧的回调。
     * @param uid User ID，从App server分配的唯一标示符。
     * @param videoTrack 接收视频track。
     * @param timeCost 耗时（毫秒）。
     * @note 该接口用于收到远端用户视频首帧的回调。
    */
    virtual void OnFirstVideoFrameReceived(const char* uid,
                                        ding::rtc::RtcEngineVideoTrack videoTrack,
                                        int timeCost) {};

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 音频首包发送回调。
     * @details 在首个音频数据包发送出去时触发此回调。
     * @param timeCost 发送耗时，从入会开始到音频首包发送出去的耗时。
     */
    virtual void OnFirstAudioPacketSend(int timeCost) {};

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 音频首包接收回调。
     * @details 在接收到远端首个音频数据包时触发此回调。
     * @param uid 远端用户ID，从App server分配的唯一标识符。
     * @param timeCost 接收耗时，从入会开始到音频首包接收到的耗时。
     */
    virtual void OnFirstAudioPacketReceived(const char* uid, int timeCost) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.2.1
     * @brief 视频渲染接收到远端视频分辨率有变化时触发此回调。
     * @param uid userId，从App server分配的唯一标示符。
     * @param videoTrack 接收视频track。
     * @param newWidth 变化后新的视频宽度
     * @param newHeight 变化后新的视频高度
     * @param oldWidth 变化前旧的视频宽度
     * @param oldHeight 变化前旧的视频高度
     * @note 视频大小可能和解码器输出的视频大小不相等，如果使用了超分一类的视频后处理。
     * @note 因为simulcast的存在，以及动态调整分辨率，横竖屏变化，视频大小和远端设置的最大推流分辨率不一定相同。
     */
    virtual void OnRenderRemoteVideoSizeChanged(const char *uid, ding::rtc::RtcEngineVideoTrack videoTrack, int newWidth, int newHeight, int oldWidth, int oldHeight) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.2.1
     * @brief 视频渲染接收到本地视频分辨率有变化时触发此回调。
     * @param videoTrack 本地视频track。
     * @param newWidth 变化后新的视频宽度
     * @param newHeight 变化后新的视频高度
     * @param oldWidth 变化前旧的视频宽度
     * @param oldHeight 变化前旧的视频高度
     * @note 视频大小可能和摄像头采集到的视频大小不相等，比如在采集之后存在缩放等处理。
     */
    virtual void OnRenderLocalVideoSizeChanged(ding::rtc::RtcEngineVideoTrack videoTrack, int newWidth, int newHeight, int oldWidth, int oldHeight) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 音频订阅情况变更回调。
     * @param uid userId，从App server分配的唯一标示符。
     * @param oldState 之前的订阅状态。
     * @param newState 当前的订阅状态。
     * @param elapseSinceLastState 两次状态变更时间间隔（毫秒）。
     * @param channel 当前频道id。
     */
    virtual void OnAudioSubscribeStateChanged(const char *uid,
                                              ding::rtc::RtcEngineSubscribeState oldState,
                                              ding::rtc::RtcEngineSubscribeState newState,
                                              int elapseSinceLastState,
                                              const char *channel) {};

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 相机流订阅情况变更回调。
     * @param uid userId，从App server分配的唯一标示符。
     * @param oldState 之前的订阅状态。
     * @param newState 当前的订阅状态。
     * @param elapseSinceLastState 两次状态变更时间间隔（毫秒）。
     * @param channel 当前频道id。
     */
    virtual void OnVideoSubscribeStateChanged(const char *uid,
                                              ding::rtc::RtcEngineSubscribeState oldState,
                                              ding::rtc::RtcEngineSubscribeState newState,
                                              int elapseSinceLastState,
                                              const char *channel) {};

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 屏幕分享流订阅情况变更回调。
     * @param uid userId，从App server分配的唯一标示符。
     * @param oldState 之前的订阅状态。
     * @param newState 当前的订阅状态。
     * @param elapseSinceLastState 两次状态变更时间间隔（毫秒）。
     * @param channel 当前频道id。
     */
    virtual void OnScreenShareSubscribeStateChanged(const char *uid,
                                                    ding::rtc::RtcEngineSubscribeState oldState, 
                                                    ding::rtc::RtcEngineSubscribeState newState,
                                                    int elapseSinceLastState,
                                                    const char *channel) {};

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 不同档次视频订阅情况变更回调。
     * @param uid userId，从App server分配的唯一标示符。
     * @param oldStreamType 之前的订阅的视频档次。
     * @param newStreamType 当前的订阅的视频档次。
     * @param elapseSinceLastState 视频档次变更时间间隔（毫秒）。
     * @param channel 当前频道id。
     * @note 订闭某个视频档次，并不表示一定能够收到该档次的视频。接收到的档次取決于推流情況以及网络情況。
     */
    virtual void OnSubscribeStreamTypeChanged(const char *uid,
                                                ding::rtc::RtcEngineVideoStreamType oldStreamType, 
                                                ding::rtc::RtcEngineVideoStreamType newStreamType, 
                                                int elapseSinceLastState, 
                                                const char *channel) {};


    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 音频采集设备音量回调。
     * @param level 音频采集设备音量值，取值[0, 100]。
     */
    virtual void OnRecordingDeviceAudioLevel(int level) {};

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 音频播放设备音量回调。
     * @param level 音频播放设备音量值，取值[0, 100]。
     */
    virtual void OnPlayoutDeviceAudioLevel(int level) {};

    /**
     * @since 3.0
     * @brief 用户音量提示回调。
     * @param speakers 用户音量信息，详见参考{@link AudioVolumeInfo}，如果 speakers 为空，则表示远端用户不发流或没有远端用户。uid为"0"表示本地说话人
     * @param speakerNumber 用户数量。
     */
    virtual void OnAudioVolumeIndication(const ding::rtc::AudioVolumeInfo* speakers, unsigned int speakerNumber) {};

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 本地音频设备状态变更。
     * @param deviceId 设备id。
     * @param deviceType  设备类型。
     * @param deviceState 设备状态。
     */
    virtual void OnAudioDeviceStateChanged(const char *deviceId, ding::rtc::RtcEngineDeviceType deviceType, ding::rtc::RtcEngineDeviceState deviceState) {};

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @brief 本地摄像头状态变更。
     * @since 3.4.1
     * @param deviceId 摄像头id。
     * @param deviceType 设备类型。
     * @param deviceState 设备状态。
     */
    virtual void OnVideoDeviceStateChanged(const char *deviceId, ding::rtc::RtcEngineDeviceType deviceType, ding::rtc::RtcEngineDeviceState deviceState) {}

#if defined(ALI_OS_ANDROID) || defined(ALI_OS_IOS)
    /**
     * @brief 音频路由发生变化回调。
     * @details 所谓“音频路由”，是指声音是从手机的扬声器还是从听筒中播放出来，音频路由变化也就是声音的播放位置发生了变化。
     * - 当音频路由为听筒时，声音比较小，只有将耳朵凑近才能听清楚，隐私性较好，适合用于接听电话。
     * - 当音频路由为扬声器时，声音比较大，不用将手机贴脸也能听清，因此可以实现“免提”的功能。
     * @param audioRouteType 当前使用的音频路由，详情参考{@link RtcEngineAudioRouteType}
     */
    virtual void OnAudioRouteChanged(ding::rtc::RtcEngineAudioRouteType audioRouteType) {}
#endif

#if defined(ALI_OS_ANDROID) 
    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.5
     * @brief 音频焦点发生变化回调。
     * @param audioFocusChange 音频焦点类型, 详细参考Android官方文档
     */
    virtual void OnAudioFocusChanged(int audioFocusChange) {}
#endif

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief API 方法已执行回调。
     * @param error 当该方法调用失败时 SDK 返回的错误码。如果该方法调用成功，SDK 将返回 0。失败，非0。
     * @param api SDK 执行的 API。
     * @param result SDK 调用 API 的调用结果。
     */
    virtual void OnApiCalledExecuted(int error, const char *api, const char *result) {};
    
    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 视频截图回调。
     * @param userId 用户id，为空或者为""代表本地用户。
     * @param videoTrack 截图视频track，参考 {@link RtcEngineVideoTrack}。
     * @param filePath 截图的本地保存路径。
     * @param width 截图宽度。
     * @param height 截图高度。
     * @param success 截图是否成功。
     * @note 该接口用于截图回调。
     */
    virtual void OnSnapshotComplete(const char* userId, ding::rtc::RtcEngineVideoTrack videoTrack, const char* filePath, int width, int height, bool success) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @brief 收到媒体扩展信息回调
     * @param uid 发送用户userId
     * @param message 扩展信息内容
     * @param size 扩展信息长度
     * @note 当一端通过 {@link RtcEngine::SendMediaExtensionMsg} 发送信息后，其他端通过该回调接收数据
     */
    virtual void OnMediaExtensionMsgReceived(const char* uid, unsigned char* message, unsigned int size) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.4
     * @brief 伴奏混音回调。
     * @param status 伴奏混音状态， 详细参考{@link ding::rtc::RtcEngineAudioMixingStatusConfig}。
     * @note 该接口用于伴奏混音状态监控。
     */
    virtual void OnAudioMixingStateChanged(const RtcEngineAudioMixingStatusConfig& status) {};

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.4
     * @brief 返回当前频道内所有的分组列表。
     * @param groups 分组列表。
     * @note 该回调发生在加入频道的时候。App通过该回调可以获得当前频道内所有的分组。后续分组变化，通过回调{@link OnAudioGroupListUpdate}获得。
     */
    virtual void OnListAllAudioGroups(const ding::rtc::StringArray &groups) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.4
     * @brief 当调用{@link JoinGroup}加入某个group，触发此回调。注意：如果JoinGroup返回非0值，那么不会产生回调。
     * @param result 0表示成功，其他值表示失败。
     * @param errMsg 如果result不为0，补充说明错误原因。
     * @param group 加入的group。
     * @param members 该group的成员用户列表。
     * @param memberCount 成员列表长度。
     */
    virtual void OnAudioGroupJoinResult(int result, const ding::rtc::String& errMsg, const ding::rtc::String& group, RtcEngineAudioGroupMember *members, int memberCount) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.4
     * @brief 当调用{@link LeaveGroup}离开某个group，触发此回调。注意：如果LeaveGroup返回非0值，那么不会产生回调。
     * @param result 0表示成功，其他值表示失败。
     * @param errMsg 如果result不为0，补充说明错误原因。
     * @param group 离开的group。
     */
    virtual void OnAudioGroupLeaveResult(int result, const ding::rtc::String& errMsg, const ding::rtc::String& group) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.4
     * @brief 当调用{@link DismissGroup}解散某个group，触发此回调。注意：如果DismissGroup返回非0值，那么不会产生回调。
     * @param result 0表示成功，其他值表示失败。
     * @param errMsg 如果result不为0，补充说明错误原因。
     * @param group 解散的group。
     */
    virtual void OnAudioGroupDismissResult(int result, const ding::rtc::String& errMsg, const ding::rtc::String& group) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.4
     * @brief 当调用{@link MixAudioToGroup}向分组或者大厅混流，触发此回调。注意：如果MixAudioToGroup返回非0值，那么不会产生回调。
     * @param group 目标分组。如果是{@link HallID}，则表示目标为大厅的音频，否则是普通分组的ID。非空。
     * @param mix true表示将本地音频流混进指定的channel audio或者分组音频中; false表示取消。
     * @param result 0表示成功，其他值表示失败。
     * @param reason 如果result不为0，用该字符串解释失败的原因。
     */
    virtual void OnAudioGroupMixResult(const ding::rtc::String& group, bool mix, int result, const ding::rtc::String& reason) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.4
     * @brief 分组中成员变化，触发此回调。
     * @param update_opt 1表示新增成员列表，-1表示离开的成员列表。
     * @param group 该回调发生的分组。
     * @param members 变动的成员列表。
     * @param memberCount "members"的长度。
     */
    virtual void OnAudioGroupMemberUpdate(int update_opt, const ding::rtc::String& group, RtcEngineAudioGroupMember *members, int memberCount) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.4
     * @brief 当前留在大厅中的所有的成员列表。
     * @param hallMembers 大厅中的成员列表。
     * @param hallMemberCount 大厅中的成员数量。
     * @note 大方会场景下，回调如果比较频繁，存在一定的cpu开销。
     * @note 留在大厅是指不在任何分组中。如果加入某个分组，意味着离开了大厅；从所有分组中离开，意味着回到了大厅。
     */
    virtual void OnAudioGroupHallMembers(RtcEngineAudioGroupMember *hallMembers, int hallMemberCount) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.4
     * @brief 分组列表变化，触发此回调。
     * @param update_opt 1表示新增的分组，-1表示删除的分组。
     * @param group 该回调指示的分组。
     */
    virtual void OnAudioGroupListUpdate(int update_opt, const ding::rtc::String& group) {}

    /**
     * @since 3.5
     * @brief 当分组名称变化后，触发此回调。
     * @param group 目标分组。不可能是{@link HallID}。
     * @param newName 分组名称。
     * @note 不论是本地修改了分组名称，还是远端修改了分组名称，都通过这个消息通知app。
     */
    virtual void OnGroupNameChanged(const ding::rtc::String& group, const ding::rtc::String& newName) {}

    /**
     * @since 3.5
     * @brief 当调用{@link SetGroupName}设置某个group名称，触发此回调。注意：如果SetGroupName返回非0值，那么不会产生此回调。
     * @param result 0表示成功，其他值表示失败。
     * @param errMsg 如果result不为0，用该字符串解释失败的原因。
     * @param group 修改名称的分组ID。
     * @param groupName 希望设置的新的命名。
     */
    virtual void OnSetGroupNameResult(int result, const ding::rtc::String& errMsg, const ding::rtc::String& group, const ding::rtc::String& groupName) {}
};

/**
 * @}
 */

/** 
 * \addtogroup Engine  引擎参考
*/
//! @{

/**
 * @interface RtcEngine
 * @brief RTC引擎的主要控制类
 */
class DING_RTC_API RtcEngine
{
protected:
    RtcEngine() {}
    virtual ~RtcEngine() {}

public:

    /**
     * @ingroup CPP_DingRtcEngineBase
     * @since 3.0
     * @brief 创建一个RtcEngine实例。
     * @param extras 用于接收来自客户设置的参数，通过JSON配置SDK的特别功能，可以是空字符串。
     * @details 该方法为同步调用，支持多实例。
     */
    static RtcEngine *Create(const char *extras);

    /**
     * @ingroup CPP_DingRtcEngineBase
     * @since 3.0
     * @brief 销毁RtcEngine实例。
     * @details 调用该方法将会释放所有内部使用的资源,当完成音视频通信之后都建议调用该方法
     *          释放实例.调用该方法后，你将不能再使用 {@link RtcEngine} 的其他方法和任何
     *          回调，如需要再次使用需要重新调用 {@link RtcEngine::Create} 方法创建一个
     *          新的实例。
     * @note 该方法为同步调用，需要等待内部资源释放之后才能执行其他方法。为避免主线程阻塞，
     *       建议开发者放在子线程调用该方法。
     * @note 为避免死锁，不建议在任何SDK的回调中调用本方法。
     */
    static void Destroy(RtcEngine *instance);

    /**
     * @brief 查询sdk当前版本号。
     * @return 当前的SDK版本号，格式为字符串，例："2.5.0"。
     * @note 在任何时刻都可以调用。
     */
    static const char *GetSDKVersion();

    /// @private
    static const char *GetSDKBuild();

    /**
     * @ingroup CPP_DingRtcEngineBase
     * @since 3.0
     * @brief 设置SDK日志文件保存路径。
     * @param logDirPath 日志文件保存目录的绝对路径，该参数是一个目录路径，而不是文件的路径。
     * @return 
     * - 0，成功
     * - 非0，失败。其中有两种错误码，释义如下：
     *   - {@link RtcEngineErrorLogRootPathError}：路径已存在或因权限问题不可访问。
     * 
     * @note 
     * - 调用此接口，请在调用其他所有SDK接口前调用本接口，避免日志出现丢失。同时，App必须保证指定的目录已存在并且可以写入。
     * - 如果，目标路径不可写入或创建日志文件失败，则会使用默认日志路径。本次设置将无效。
     * - 默认路径：windows平台，在用户目录下dingding/log。
     */
    static int SetLogDirPath(const char *logDirPath);

    /**
     * @ingroup CPP_DingRtcEngineOthers
     * @since 3.0
     * @brief 获取当前SDK日志文件保存路径，包含SDK添加的子目录。
     * @param buf 用于接收目录，不允许NULL。
     * @param maxBufSize buf的大小。拷贝时包含字符串的结束符'\0'。
     * @return
     * - true，成功，路径存放在buf中；
     * - false，失败，一种可能是提供的buf长度过短，放不下；另一种可能是尚未设置日志路径。
     */
    static bool GetLogDirPath(char *buf, int maxBufSize);

    /**
     * @ingroup CPP_DingRtcEngineOthers
     * @since 3.0
     * @brief 设置日志等级。
     * @param logLevel Log级别，详见{@link RtcEngineLogLevel}枚举类型。
     * @return
     * - 0：成功；
     * - 非0：失败。比如没有先调用SetLogDirPath。
     *
     * @note 设置 SDK 的输出日志等级。默认值为 {@link RtcEngineLogLevel::RtcEngineLogLevelInfo}。调用此接口，请先于SetLogDirPath接口调用之前调用本接口，否则可能不起作用。如果设置为RtcEngineLogLevelNone，那么停止日志输出。
     */
    static int SetLogLevel(ding::rtc::RtcEngineLogLevel logLevel);

    /**
     * @ingroup CPP_DingRtcEngineOthers
     * @since 3.0
     * @brief 根据错误码，获取错误码描述，也适用于获取 Warning 的描述。
     * @param error_code 错误码，详见{@link RtcEngineErrorCode}。
     * @return 错误码描述字符串。
     * @note 在任何时刻都可以调用。
     */
    static const char* GetErrorDescription(int error_code);

    /**
     * @ingroup CPP_DingRtcEngineBase
     * @since 3.0
     * @brief 设置相关监听事件回调。
     * @details {@link RtcEngineEventListener}类是SDK向应用发送自身回调事件的通知,
     *          应用开发者需要实现该接口类的方法来获取相关事件通知。默认所有接口都有空实现，
     *          开发者不需要实现所有方法，可根据自身业务来实现对应的事件方法。
     * @param listener  用于接收来自engine的消息。
     * @note 所有的回调方法都是SDK内部的线程，不可以做UI操作，有UI相关操作需要抛主线程操作。
     * @note 不建议在回调方法中做耗时操作，如调用RtcEngine本身的Destroy方法，这将有可能
     *       引起不必要的阻塞从而影响SDK的运行。
     */
    virtual int SetEngineEventListener(RtcEngineEventListener *listener) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudio
     * @since 3.4
     * @brief 获取伴奏混音管理类接口。
     * @return 音频伴奏混音管理类接口。
     */
    virtual RtcEngineAudioMixingManager* GetAudioMixingManager() = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudio
     * @since 3.0
     * @brief 获取音频设备管理类接口。
     * @return 音频设备管理类接口。
     */
    virtual RtcEngineAudioDeviceManager* GetAudioDeviceManager() = 0;


    /**
     * @ingroup CPP_DingRtcEngineVideo
     * @since 3.0
     * @brief 获取视频设备管理类接口。
     * @return 视频设备管理类接口。
     */
    virtual RtcEngineVideoDeviceManager* GetVideoDeviceManager() = 0;

    /**
     * @ingroup CPP_DingRtcEngineWb
     * @since 3.6
     * @brief 获取白板管理类接口。
     * @return 白板管理类接口。
     */
    virtual RtcEngineWbManager* GetWbManager() = 0;

    /**
     * @ingroup CPP_DingRtcEngineOthers
     * @since 3.0
     * @brief 获取当前网络链接状态。
     * @return 当前链接状态，参考{@link RtcEngineConnectionStatus}。
     * @note 网络链接状态发生变化时，SDK会通过{@link RtcEngineEventListener::OnConnectionStatusChanged}回调通知。
     */
    virtual RtcEngineConnectionStatus GetCurrentConnectionStatus() = 0;

    /**
     * @ingroup CPP_DingRtcEngineChannel
     * @since 3.0
     * @brief 该方法让用户加入频道，一个频道内可以加入多个用户进行互相通话。
     * @param authInfo    认证信息，从App Server获取。
     * @param userNameUtf8 任意用于显示的用户名称。不是User ID。支持emoji。要求utf8编码。如何转换为utf8字符串，参考<a href="https://help.aliyun.com/document_detail/2664101.html?#f08191615e5am">客户端入会失败常见问题</a>
     * @return
     * - 0：成功；
     * - <0：失败。
     * @note
     *    - 异步接口。是否成功入会，通过 {@link RtcEngineEventListener::OnJoinChannelResult}判断。如果接口返回值为非0，则不会触发回调。
     *    - 正常情况一个Appid对应一个App，只有同一个AppId的应用可进行互相童话，不同AppId和channelID不同都不能互通。
     *    - 当入会成功后，会触发远端用户收到 {@link RtcEngineEventListener::OnJoinChannelResult} 回调。
     *    - 如果已在会中想重新入会，需要先调用 {@link LeaveChannel} 离会，且确保 {@link RtcEngineEventListener::OnLeaveChannelResult} 回调收到后，再次调用进行入会。
     *    - userNameUtf8如果过长，超过64个字节，调用会失败。App业务系统应当避免过长的userNameUtf8传入进来。一般汉字占用3个字节，简单的emoji占用4~8个字节，一个复杂emoji占用34甚至更多字节。
     */
    virtual int JoinChannel(const ding::rtc::RtcEngineAuthInfo &authInfo, const char *userNameUtf8) = 0;

    /**
     * @ingroup CPP_DingRtcEngineChannel
     * @since 3.0
     * @brief  离开频道，即挂断或者停止通话。
     * @details 如果当前不在频道内，{@link LeaveChannel} 不会有任何影响。{@link LeaveChannel} 会产生消息通知频道内其他用户。
     * @details 调用 {@link JoinChannel}  后必须要调用 {@link LeaveChannel} 离开频道，否则无法进入下一个频道通话。
     * @details 异步方法，成功调用后需要收到 {@link RtcEngineEventListener::OnLeaveChannelResult} 才会真正退出频道，离开频道成功远端用户会触发会收到 {@link RtcEngineEventListener::OnRemoteUserOnLineNotify}回调。
     * @return
     * - 0：成功；
     * - 非0：失败。
     * @note
     *  - 如果调用了 {@link LeaveChannel} 后立刻调用 {@link Destroy} 方法，SDK将会无法触发 {@link RtcEngineEventListener::OnLeaveChannelResult}。
     */
    virtual int LeaveChannel() = 0;

    /**
     * @ingroup CPP_DingRtcEngineUserInfo
     * @since 3.0
     * @brief 获取远端在线用户列表。
     * @param userList 当前频道内所有在线用户userid列表。
     */
    virtual void GetOnlineRemoteUsers(StringArray &userList) = 0;

    /**
     * @ingroup CPP_DingRtcEngineUserInfo
     * @since 3.0
     * @brief 查询远端用户状态。        
     * @details 通过 uid 查询，指定远端用户的状态信息，参数dict类型为标准std::map<std::string， std::string>容器，通过键值对的方式进行访问。 
     * @param uid 用户ID。从App server分配的唯一标示符。
     * @param dict App提供的容器，用于存放用户数据。
     * @return 
     * - 0：成功；
     * - 非0：失败，返回值为具体的错误码。
     * @note Key信息： 
    *  - userID：远端用户的ID。
     *  - isOnline：远端用户是否在线。
     *  - displayName：远端用户的名称。
     *  - hasAudio：远端用户是否推送了音频流。
     *  - hasCamera：远端用户是否推送了相机流。
     *  - hasScreenSharing：远端用户是否推送了屏幕流。
     *  - requestAudio：本端是否订阅了此远端用户的音频流。
     *  - requestCamera：本端是否订阅了此远端用户的相机流。此状态，系在成功订阅该用户摄像头流前的状态。
     *  - requestScreenSharing：本端是否订阅了此远端用户的屏幕流。此状态，系在成功订阅该用户屏幕共享前的状态。
     *  - preferCameraType：如果网络带宽允许，优先订阅的相机流的大小规格，选择范围：FHD，HD，SD，LD。
     *  - subScribedAudio：本端是否拉到了此远端用户的音频流。
     *  - subScribedCameraType：本端拉到了此远端用户的相机流的大小规格，可能的值：FHD，HD，SD，LD。 
     *  - subScribedScreenSharing：本端是否拉到了此远端用户的屏幕流。
     *  - hasCameraView：本端是否设置了此远端用户相机流的view。
     *  - hasScreenView：本端是否设置了此远端用户屏幕流的view。
     *  - isCameraMirror：本端是否设置了此远端用户的相机流为镜像。
     *  - isScreenMirror：本端是否设置了此远端用户的屏幕流为镜像。
     */
    virtual int GetUserInfo(const char *uid, Dictionary &dict) = 0;

    /**
     * @ingroup CPP_DingRtcEngineUserInfo
     * @since 3.0
     * @brief 查询用户是否在线。
     * @param uid 用户ID，从App server分配的唯一标示符。不可为NULL。
     * @return
     *  - true：用户在线；
     *  - false：用户不在线。
     * @note 根据特定的UserID，查询该用户是否在线。
     */
    virtual bool IsUserOnline(const char *uid) = 0;

    /**  
     * @ingroup CPP_DingRtcEngineChannel
     * @since 3.0
     * @brief 获取状态：当前是否在频道中。
     * @return 
     - - true：表示当前已经加入了频道；
     * - false：表示当前不在频道中。
     * @note 该方法可在加入频道前或加入频道后调用。
     */
    virtual bool IsInCall() = 0;

    /**
     * @ingroup CPP_DingRtcEnginePub
     * @since 3.0
     * @brief 是否推送本地视频（摄像头）流。
     * @param enable 
     * - 是否开启/关闭本地视频流推送。
     * - true：开启视频流推送。
     * - false：关闭视频流推送。
     * @return
     * - 0：设置成功；
     * - <0：设置失败，返回错误码。
     * @note SDK默认设置不推送视频流，在加入频道前也可以调用此接口修改默认值，并在加入频道成功时生效。
     */
    virtual int PublishLocalVideoStream(bool enable) = 0;

    /**
     * @ingroup CPP_DingRtcEnginePub
     * @since 3.0
     * @brief 是否推送本地音频流。
     * @param enable 
     * - 是否开启/关闭本地音频流推送。
     * - true：开启音频流推送。
     * - false：关闭音频流推送。
     * @return
     * - 0：设置成功；
     * - <0：设置失败，返回错误码。
     * @note SDK默认设置不推送音频流，在加入频道前也可以调用此接口修改默认值，并在加入频道成功时生效。
     */
    virtual int PublishLocalAudioStream(bool enable) = 0;

    /**
     * @ingroup CPP_DingRtcEnginePub
     * @since 3.0
     * @brief 查询是否推送视频流。
     * @return
     * - true：会推送；
     * - false：不会推送。
     */
    virtual bool IsLocalVideoStreamPublished() = 0;

    /**
     * @ingroup CPP_DingRtcEngineScreenShare
     * @since 3.0
     * @brief 是否推送屏幕分享。
     * @return
     * - true：会推送；
     * - false：不会推送。
     * @note SDK默认设置不推送屏幕分享，如果需要推送屏幕共享，可以通过{@link StartScreenShareByDesktopId}， {@link StartScreenShareByWindowId}方法开启屏幕共享推送。
     */
    virtual bool IsScreenSharePublished() = 0;

    /**
     * @ingroup CPP_DingRtcEnginePub
     * @since 3.0
     * @brief 是否推送音频流。
     * @return
     * - true：会推送；
     * - false：不会推送。
     */
    virtual bool IsLocalAudioStreamPublished() = 0;

    /**
     * @ingroup CPP_DingRtcEngineVideo
     * @since 3.0
     * @brief 暂停/恢复本地视频流的发送。
     * @param mute
     *          - true表示暂停视频流的发送。
     *          - false表示恢复视频流的发送。
     * @param track 只支持{@link RtcEngineVideoTrackCamera 和 RtcEngineVideoTrackScreen}。
     * @return
     * - 0：成功；
     * - 非0：失败。
     * @note 视频流暂停发送时，采集和编码模块仍然在工作。
     */
    virtual int MuteLocalVideo(bool mute, ding::rtc::RtcEngineVideoTrack track) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudio
     * @since 3.0
     * @brief 静音/取消静音本地音频数据。
     * @param mute 
     *         - true表示静音本地音频，发送静音帧。
     *         - false表示取消静音。
     * @return 
     * - 0：成功；
     * - 非0：失败。
     * @note mute状态会保持，除非再次调用MuteLocalAudio更新，或者重新创建engine instance。
     */

    virtual int MuteLocalAudio(bool mute) = 0;
    
    /**
     * @ingroup CPP_DingRtcEngineScreenShare
     *
     * @since 3.0
     * @brief 根据桌面ID进行屏幕分享。
     * @param desktopId 桌面ID（可通过GetScreenShareSourceInfo接口获取）。
     * @param config 屏幕分享配置，可指定分享区域，详见 {@link RtcEngineScreenShareConfig}。
     * @return
     * - 0：成功；
     * - <0：失败。
     */
    virtual int StartScreenShareByDesktopId(const char* desktopId, const ding::rtc::RtcEngineScreenShareConfig& config) = 0;

    /**
     * @ingroup CPP_DingRtcEngineScreenShare
     *
     * @since 3.0
     * @brief 根据窗口ID进行屏幕分享。
     * @param windowId 窗口ID（可通过GetScreenShareSourceInfo接口获取）。
     * @param config 屏幕分享配置（暂不支持指定区域分享）。
     * @return
     * - 0：成功；
     * - <0：失败。
     * @note 自3.5.0版本起，根据窗口ID进行屏幕分享支持指定区域分享。
     */
    virtual int StartScreenShareByWindowId(const char* windowId, const ding::rtc::RtcEngineScreenShareConfig& config) = 0;
    
    /**
     * @ingroup CPP_DingRtcEngineScreenShare
     *
     * @since 3.4.0
     * @brief 更新当前正在进行中的屏幕分享的配置。
     * @param config 屏幕分享配置，详见 {@link RtcEngineScreenShareConfig}。
     * @return
     *  - 0： 设置成功
     *  - <0： 失败。
     * @note 正在屏幕分享中才有效。桌面分享可进行更新，自3.5.0版本起，窗口分享支持指定区域分享。
    */
    virtual int UpdateScreenShareConfig(const ding::rtc::RtcEngineScreenShareConfig& config) = 0;

#if defined(ALI_OS_IOS)
    /**
     * @brief iOS屏幕分享。
     * @param appGroup 插件和宿主app约定使用相同的appGroup。
     * @param mode 屏幕分享配置，详见 {@link RtcScreenShareMode}。
     * @return
     * - 0：成功；
     * - <0：失败。
     */
   virtual int StartScreenShare(const String& appGroup, ding::rtc::RtcScreenShareMode mode) = 0;
#endif

#if defined(ALI_OS_ANDROID)
    /**
     * @brief android屏幕分享。
     * @param intent 外部创建启动屏幕分享的Activity。
     * @return
     * - 0：成功；
     * - <0：失败。
     */
   virtual int StartScreenShare(void* intent, ding::rtc::RtcScreenShareMode mode) = 0;
#endif

    /**
     * @ingroup CPP_DingRtcEngineScreenShare
     * @since 3.0
     * @brief 停止屏幕分享。
     * @return
     * - 0：成功；
     * - <0：失败。
     */
    virtual int StopScreenShare() = 0;

    /**
     * @ingroup CPP_DingRtcEngineScreenShare
     * @since 3.0
     * @brief 获取屏幕分享源信息。
     * @param sourceType 屏幕分享类型。
     * @return RtcEngineScreenDeviceInfoList 返回屏幕共享源列表，详见 {@link ding::rtc::RtcEngineScreenDeviceInfoList}。
     * @note 请在遍历完屏幕共享源列表后调用其release成员方法，由SDK内部释放相关资源。
     */
    virtual RtcEngineScreenDeviceInfoList* GetScreenShareSourceInfo(ding::rtc::RtcEngineScreenShareType sourceType) = 0;

    /**
     * @ingroup CPP_DingRtcEngineScreenShare
     * @since 3.0
     * @brief 获取当前屏幕共享源Id。
     * @return 当前屏幕共享源Id。
     */
    virtual unsigned int GetCurrentScreenShareSourceId() = 0;

    /**
     * @since 3.0
     * @brief 获取当前屏幕共享源类型。
     * @return RtcEngineScreenShareType 当前屏幕共享源类型，详见 {@link ding::rtc::RtcEngineScreenShareType}。
     */
    virtual RtcEngineScreenShareType GetCurrentScreenShareSourceType() = 0;

    /**
     * @ingroup CPP_DingRtcEngineSub
     * @since 3.0
     * @brief 停止/恢复订阅所有远端音频流，包含后入会成员的订阅。从3.4开始，“所有人”是指大厅中所有人，解释见note。
     * @param sub 
     * - 是否订阅所有远端音频流。
     * - true：订阅所有用户的音频流，后入会成员会自动订阅其音频。
     * - false：停止订阅所有用户的音频流，后入会成员也不会自动订阅其音频。
     * @return
     * - 0：设置成功；
     * - 非0：设置失败。
     * @note
     * - 入会前入会后均可以调用。
     * @note
     * - 从3.4开始，引入音频分组，这个接口的操作是针对大厅的音频。所有人是指留在大厅中的所有人，不包括加入分组中的人。如果您的产品中未使用音频分组，那么行为兼容以前的定义不变。如果使用了分组，想要订阅某个分组的音频，请调用新接口{@link SwitchSubscriptionToGroup}。
     * - 如果local user加入了某个分组，您坚持使用SubscribeAllRemoteAudioStreams接口来订阅音频，则会失败，原因是local user不在大厅中，根据权限约束，不允许订阅大厅音频。
     * - 取消订阅SubscribeAllRemoteAudioStreams(false)不受权限约束，总是可以退订任何在订阅的音频。您也可以使用{@link SwitchSubscriptionToGroup}(groupid="")来达到同样的结果。
     */
    virtual int SubscribeAllRemoteAudioStreams(bool sub) = 0;

    /**
     * @ingroup CPP_DingRtcEngineSub
     * @since 3.0
     * @brief 停止/恢复订阅所有远端视频流，包含后入会成员的订阅。
     * @param sub 
     * - 是否订阅所有远端视频流。
     * - true：订阅所有用户的视频流，后入会成员会自动订阅其视频。
     * - false：停止订阅所有用户的视频流，后入会成员也不会自动订阅其视频。
     * @return
     * - 0：设置成功；
     * - 非0：设置失败。
     * @note
     * - 入会前入会后均可以调用。
     * - 如果设置了 SubscribeAllRemoteVideoStreams(false)，会取消订阅已订阅的远端视频流，同时后面再入会的人不会进行自动订阅；但是依然可以通过 {@link SubscribeRemoteVideoStream} 来主动订阅某个人的远端视频流。
     */
    virtual int SubscribeAllRemoteVideoStreams(bool sub) = 0;

    /**
     * @ingroup CPP_DingRtcEngineSub
     * @since 3.0
     * @brief 停止/恢复订阅远端用户的视频流。
     * @param uid 用户ID，从App server分配的唯一标示符。
     * @param track 视频流类型{@link RtcEngineVideoTrack}。
     * @param sub 
     * - 是否订阅远端用户的视频流。
     * - true：订阅指定用户的视频流。
     * - false：停止订阅指定用户的视频流。
     * @return
     * - 0：设置成功；
     * - 非0：设置失败。
     * @note
     * - 如果设置了 SubscribeAllRemoteVideoStreams(false)，会取消订阅已订阅的远端视频流，同时后面再入会的人不会进行自动订阅；但是依然可以通过 {@link SubscribeRemoteVideoStream} 来主动订阅某个人的远端视频流。
     */
    virtual int SubscribeRemoteVideoStream(const char* uid, ding::rtc::RtcEngineVideoTrack track, bool sub) = 0;

    /**
     * @ingroup CPP_DingRtcEngineSub
     * @since 3.0
     * @brief 设置订阅视频流分辨率规格。
     * @param uid  用户ID，从App server分配的唯一标示符。
     * @param streamType 视频流的分辨率规格。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int SetRemoteVideoStreamType(const char* uid, ding::rtc::RtcEngineVideoStreamType streamType) = 0;

    /**
     * @ingroup CPP_DingRtcEngineSub
     * @since 3.0
     * @brief 设置默认订阅的视频流分辨率规格；如果不设置，则默认订阅最大的分辨率规格。
     * @param streamType 视频流的分辨率规格。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int SetRemoteDefaultVideoStreamType(ding::rtc::RtcEngineVideoStreamType streamType) = 0;

    /**
     * @ingroup CPP_DingRtcEngineVideo
     * @since 3.0
     * @brief 禁用或启用本地视频采集。
     * @param enabled
     *          - true ：启用本地视频采集。
     *          - false ：禁用本地视频采集。
     * @return
     * - 0：成功；
     * - 非0：失败。
     * @note 默认为开启状态。
     */
    virtual int EnableLocalVideo(bool enabled) = 0;

    /**
     * @ingroup CPP_DingRtcEngineMediaEngine
     * @brief 开启或关闭视频美颜。
     * @param enable 是否开启。
     * @param options 视频美颜参数。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int EnableBeautyFace(bool enable, ding::rtc::RtcEngineBeautyFaceOptions options) = 0;
    
    /**
     * @ingroup CPP_DingRtcEngineMediaEngine
     * @brief 开启或关闭虚拟背景。
     * @param enable 是否开启。
     * @param options 虚拟背景参数。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int EnableVirtualBackground(bool enable, ding::rtc::RtcEngineVirtualBackgroundOptions options) = 0;

    /**
     * @ingroup CPP_DingRtcEngineMediaEngine
     * @brief 设置视频暗光增强功能。
     * @param options 视频暗光增强参数。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int SetVideoEnhance(ding::rtc::RtcEngineVideoEnhanceOptions options) = 0;

    /**
     * @ingroup CPP_DingRtcEngineMediaEngine
     * @brief 设置视频去噪场景功能。
     * @param options 视频去噪参数。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int SetVideoDenoise(ding::rtc::RtcEngineVideoDenoiseOptions options) = 0;

    /**
     * @ingroup CPP_DingRtcEngineMediaEngine
     * @since 3.4
     * @brief 设置音频降噪功能。
     * @param options 音频降噪参数。
     * @return
     * - 0：成功；
     * - 非0：失败。
     * @remark music mode，自动关闭降噪。
     */
    virtual int SetAudioDenoise(ding::rtc::RtcEngineAudioDenoiseOptions options) = 0;

    /**
     * @ingroup CPP_DingRtcEngineVideo
     * @since 3.0
     * @brief 设置相机流视频编码属性。
     * @details 该方法用于设置相机流视频编码属性对应的视频参数，如分辨率、帧率、码率、视频方向等所有设置的参数都有相应的范围限制，如果设置的参数不在有效范围内，SDK会自动调节。
     * @param config 预定义的编码属性，详见{@link ding::rtc::RtcEngineVideoEncoderConfiguration}。
     * @note
     * - 该方法在入会前和入会后都可以调用，如果每次入会只需要设置一次相机流视频编码属性，建议在入会前调用。
     */
    virtual void SetVideoEncoderConfiguration(const ding::rtc::RtcEngineVideoEncoderConfiguration& config) = 0;

    /**
     * @ingroup CPP_DingRtcEngineScreenShare
     * @since 3.0
     * @brief 设置屏幕流视频编码属性。
     * @details 该方法用于设置屏幕流视频编码属性对应的视频参数，如分辨率、帧率、码率、视频方向等所有设置的参数都有相应的范围限制，如果设置的参数不在有效范围内，SDK会自动调节。
     * @param config 预定义的屏幕共享编码属性，详见{@link ding::rtc::RtcEngineScreenShareEncoderConfiguration}。
     * @note
     * - 该方法在入会前和入会后都可以调用，如果每次入会只需要设置一次屏幕流视频编码属性，建议在入会前调用。
     */
    virtual void SetScreenShareEncoderConfiguration(const ding::rtc::RtcEngineScreenShareEncoderConfiguration& config) = 0;

    /**
     * @ingroup CPP_DingRtcEngineVideo
     * @since 3.0
     * @brief 设置本地预览显示视图。
     * @param renderConfig 本地视频显示属性。
     * @param track  只支持 {@link RtcEngineVideoTrackCamera} 和 {@link RtcEngineVideoTrackScreen}。
     * @return
     * - 0：成功；
     * - 非0：失败。
     * @note 支持joinChannel之前和之后切换窗口。如果 {@link RtcEngineVideoCanvas::displayView} 为null，则停止显示。
     *       如果需要重新设置{@link RtcEngineVideoCanvas::renderMode}，请保持renderConfig中其他属性不变，仅修改
     *       {@link RtcEngineVideoCanvas::renderMode}。
     *       如果需要重新设置{@link RtcEngineVideoCanvas::mirrorMode}，请保持renderConfig中其他属性不变，仅修改
     *       {@link RtcEngineVideoCanvas::mirrorMode}。
     */
    virtual int SetLocalViewConfig(const ding::rtc::RtcEngineVideoCanvas& renderConfig, ding::rtc::RtcEngineVideoTrack track) = 0;

    /**
     * @ingroup CPP_DingRtcEngineVideo
     * @since 3.0
     * @brief 设置远端视频显示视图。
     * @param renderConfig 远端视频显示属性。
     * @param uid user id。
     * @param track 只支持 {@link RtcEngineVideoTrackCamera} 和 {@link RtcEngineVideoTrackScreen}。
     * @return
     * - 0：成功；
     * - 非0：失败。
     * @note 如果 {@link RtcEngineVideoCanvas::displayView}为null，则停止显示。
     *       如果需要重新设置 {@link RtcEngineVideoCanvas::renderMode}，请保持renderConfig中其他属性不变，仅修改
     *        {@link RtcEngineVideoCanvas::renderMode}。
     *       如果需要重新设置 {@link RtcEngineVideoCanvas::mirrorMode}，请保持renderConfig中其他属性不变，仅修改
     *        {@link RtcEngineVideoCanvas::mirrorMode}。
     */
    virtual int SetRemoteViewConfig(const ding::rtc::RtcEngineVideoCanvas& renderConfig, const char *uid, ding::rtc::RtcEngineVideoTrack track) = 0;

    /**
     * @ingroup CPP_DingRtcEngineVideo
     * @since 3.0
     * @brief 更新渲染配置。
     * @param renderConfig config包含了窗口以及渲染方式。
     * @param uid user id，为 "" 代表本地用户。
     * @param track 只支持 {@link RtcEngineVideoTrackCamera} 和 {@link RtcEngineVideoTrackScreen}。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int UpdateViewConfig(const ding::rtc::RtcEngineVideoCanvas& renderConfig, const char *uid, RtcEngineVideoTrack track) = 0;

    /// @private
    virtual int AddLocalVideoSink(ding::rtc::RtcEngineVideoStreamSink *sink, ding::rtc::RtcEngineVideoTrack track) = 0;
    /// @private
    virtual int RemoveLocalVideoSink(ding::rtc::RtcEngineVideoStreamSink *sink, ding::rtc::RtcEngineVideoTrack track) = 0;
    /// @private
    virtual int AddRemoteVideoSink(ding::rtc::RtcEngineVideoStreamSink *sink, const char *uid, ding::rtc::RtcEngineVideoTrack track) = 0;
    /// @private
    virtual int RemoveRemoteVideoSink(ding::rtc::RtcEngineVideoStreamSink *sink, const char *uid, ding::rtc::RtcEngineVideoTrack track) = 0;

    /**
     * @ingroup CPP_DingRtcEnginePreview
     * @since 3.0
     * @brief 开启本地视频预览。
     * @return
     * - 0：成功；
     * - 非0：失败。
     * @note 如果没有调用 {@link SetLocalViewConfig} 设置显示的view，则无法预览。
     *       可以在{@link JoinChannel}之前就开启预览会自动打开摄像头。
     *       调用LeaveChannel不会关闭本地预览，会一直保留，直到调用StopPreview结束。
     */
    virtual int StartPreview() = 0;

    /**
     * @ingroup CPP_DingRtcEnginePreview
     * @since 3.0
     * @brief 关闭本地视频预览。
     * @return
     * - 0：成功；
     * - 非0：失败。
     * @note {@link LeaveChannel} 会自动停止本地预览；如果没有在推相机流，则会自动关闭摄像头。
     */
    virtual int StopPreview() = 0;

    /**
     * @ingroup CPP_DingRtcEngineVideo
     * @since 3.0
     * @brief 注册视频帧观察类对象。
     * @param observer 视频观察类对象指针。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int RegisterVideoFrameObserver(ding::rtc::RtcEngineVideoFrameObserver* observer) = 0;

    /**
     * @ingroup CPP_DingRtcEngineVideo
     * @since 3.1
     * @brief 设置视频帧数据观察开关。
     * @param enabled 是否允许数据回调。
     * @param position 视频数据类型，详见 {@link RtcEngineVideoObservePosition}。
     * @return 0：success。
     */
    virtual int EnableVideoFrameObserver(bool enabled, unsigned int position) = 0;

   /**
    * @ingroup CPP_DingRtcEngineAudio
    * @since 3.0
    * @brief 设置音频profile。
    * @param audioProfile 音频采集/编码模式参数， 详见{@link RtcEngineAudioProfile}。
    * @param audioScene 音频场景模式参数，详见{@link RtcEngineAudioScenario}。
    * @return 
    * - 0：成功；
    * - 其他：失败。
    * @note 只支持 {@link JoinChannel} 前设置有效。
    * @note 缺省RtcEngineHighQualityMode模式。
    */
    virtual int SetAudioProfile(int audioProfile, int audioScene) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudio
     * @since 3.0
     * @brief 设置音量回调频率和平滑系数。
     * @param interval 时间间隔，单位毫秒，最小值不得小于100ms, 建议设置300-500ms, <= 0表示不启用音量提示和说话人提示功能。
     * @param smooth 平滑系数，数值越大平滑程度越高，反之越低，实时性越好，建议设置3，范围[0, 9]。
     * @param reportVad 
     * - 说话人检测开关。
     * - 1：开启。
     * - 0：关闭。
     * @return
     * - 0：成功；
     * - <0：失败。
     */
    virtual int EnableAudioVolumeIndication(int interval, int smooth, int reportVad) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudio
     * @since 3.0
     * @brief 注册音频观察类对象。
     * @param observer 音频观察类对象指针。
     * @return
     * - 0：成功；
     * - 非0：失败。
     * @note 要移除observer，调用RegisterAudioFrameObserver(NULL)。
     */
    virtual int RegisterAudioFrameObserver(ding::rtc::RtcEngineAudioFrameObserver* observer) = 0;

    /**
     * @brief 订阅音频数据。
     * @param enabled 是否允许数据回调。
     * @param position 音频裸数据源类型，详见 {@link RtcEngineAudioObservePosition}。
     * @return 0：success。
     */
    virtual int EnableAudioFrameObserver(bool enabled, unsigned int position) = 0;

    /**
     * @brief 订阅音频数据。
     * @param enabled 是否允许数据回调。
     * @param position 音频裸数据源类型详见 {@link RtcEngineAudioObservePosition}。
     * @param config 音频裸数据配置, 详见 {@link RtcEngineAudioFrameObserverConfig}。
     * @return 0：success。
     */
    virtual int EnableAudioFrameObserver(bool enabled, unsigned int position, const ding::rtc::RtcEngineAudioFrameObserverConfig& config) = 0;

    /**
     * @brief 订阅音频数据。
     * @param enabled 是否允许数据回调。
     * @param audioSource 音频裸数据源类型，详见 {@link RtcEngineAudioSource}。
     * @return 0：success。
     * @deprecated 该方法已废弃，请使用 EnableAudioFrameObserver(bool, unsigned int) 代替。
     */
    DING_RTC_DEPRECATED virtual int EnableAudioFrameObserver(bool enabled, RtcEngineAudioSource audioSource) = 0;

    /**
    * @since 3.4.2
    * @brief 设置算法或者其他私有参数。
    * @param params 用于接收来自客户设置的参数，通过JSON配置SDK的特别功能。
    * @return 
    * - 0：成功；
    * - 其他：失败。
    */
    virtual int SetParameters(const char* params) = 0;

    /* 通过渲染检查引擎是否工作正常
     */
    /// @private
    virtual void EnableDetailedStreamObserver(bool enable) = 0;
    /// @private
    virtual bool GetRemoteStreamRendererDetail(const char *uid,
        RtcEngineVideoTrack track,
        VideoRenderDetailInfo &info) = 0;
    /// @private
    virtual bool GetLocalVideoRendererDetail(ding::rtc::RtcEngineVideoTrack track, ding::rtc::VideoRenderDetailInfo &info) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudio
     * @since 3.0
      * @brief 设置外部音频源参数。
      * @param enable 
      *          - true：开启。
      *          - false：关闭。
      * @param sampleRate 采样率（Hz），取值：8000，16000，32000，44100，48000。
      * @param channels 声道数，取值：1：单声道；2：双声道。
      * @return
      * - >=0：成功；
      * - <0：失败。
      */
      virtual int SetExternalAudioSource(bool enable, int sampleRate, int channels) = 0;

      /**
       * @ingroup CPP_DingRtcEngineAudio
       * @since 3.0
       * @brief 推送外部音频采集源。
       * @param frame 外部音频帧，详见 {@link ding::rtc::RtcEngineAudioFrame}。
       * @return
       * - 0：成功；
       * - <0：失败。
       */
      virtual int PushExternalAudioFrame(ding::rtc::RtcEngineAudioFrame* frame) = 0;

      /**
       * @ingroup CPP_DingRtcEngineAudio
       * @since 3.1
       * @brief 设置是否启用外部输入音频播放。
       * @param enable 是否开启。
       * - true： 开启。
       * - false： 关闭。
       * @param sampleRate 采样率（Hz)，取值：8000, 16000, 32000, 44100, 48000。
       * @param channels 声道数 1：单声道; 2：双声道。
       * @return
       * - 0： 成功。
       * - 非0： 失败。
      */
      virtual int SetExternalAudioRender(bool enable, int sampleRate, int channels) = 0;

      /**
       * @ingroup CPP_DingRtcEngineAudio
       * @since 3.1
       * @brief 推送外部音频到扬声器。
       * @param frame 外部音频帧，详见 {@link ding::rtc::RtcEngineAudioFrame}。
       * @return
       * - 0： 成功。
       * - <0： 失败。
      */
      virtual int PushExternalAudioRenderFrame(ding::rtc::RtcEngineAudioFrame* frame) = 0;

      /**
       * @ingroup CPP_DingRtcEngineAudio
       * @since 3.2
       * @brief 开启自定义音频采集，开始后，SDK内部不再操作音频采集设备。
       * @param enable 是否开启。
       * @note 需要您在进入房间前设置才能生效，暂不支持进入房间后再设置。
       * @note 使用自定义采集时， 不能再使用外部音频输入源{@link SetExternalAudioSource}相关功能。
       * @return
       * - 0： 成功。
       * - <0： 失败。
      */
      virtual int EnableCustomAudioCapture(bool enable) = 0;
      
      /**
       * @ingroup CPP_DingRtcEngineAudio
       * @since 3.2
       * @brief 向SDK投送自己采集的音频数据。
       * @param frame 音频帧，详见 {@link ding::rtc::RtcEngineAudioFrame}。
       * @note 音频帧数据只支持16位PCM格式，10ms帧长，采样率支持48000/44100/32000/16000，声道数支持单声道/双声道。
       * @note frame buffer的创建和销毁由app负责管理。
       * @return
       * - 0：成功。
       * - <0：失败。
      */
      virtual int SendCustomAudioCaptureFrame(RtcEngineAudioFrame* frame) = 0;

      /**
       * @ingroup CPP_DingRtcEngineAudio
       * @since 3.2
       * @brief 开启自定义音频播放，开始后，SDK内部不再操作音频播放设备。
       * @param enable 是否开启。
       * @note 需要您在进入房间前设置才能生效，暂不支持进入房间后再设置。
       * @note 使用自定义播放时，不能再使用外部音频渲染源{@link SetExternalAudioRender}相关功能。
       * @return
       * - 0： 成功。
       * - <0： 失败。
      */
      virtual int EnableCustomAudioRender(bool enable) = 0;

      /**
       * @ingroup CPP_DingRtcEngineAudio
       * @since 3.2
       * @brief 获取可播放的音频数据。
       * @param frame 音频帧，详见 {@link ding::rtc::RtcEngineAudioFrame}。
       * @note 音频帧buffer需要使用者申请释放，音频帧数据只支持16位PCM格式，10ms帧长，采样率支持48000/44100/32000/16000，声道数支持单声道/双声道。
       * @note frame buffer的创建和销毁由app负责管理。
       * @return
       * - 0： 成功。
       * - <0： 失败。
      */
      virtual int GetCustomAudioRenderFrame(RtcEngineAudioFrame* frame) = 0;

      /**
       * @since 3.1
       * @brief 设置音频播放音量，音量范围[0, 100]。
       * @param volume 音量值。
       * @return
       * - 0：成功；
       * - 非0：失败。
       */
      virtual int SetPlayoutSignalVolume(int volume) = 0;

      /**
       * @ingroup CPP_DingRtcEngineAudio
       * @since 3.1
       * @brief 获取音频播放音量。
       * @return 音量。
       */
      virtual int GetPlayoutSignalVolume() = 0;

      /**
       * @since 3.5
       * @brief 设置音频采集音量，音量范围[0, 100]。
       * @param volume 音量值。
       * @return
       * - 0：成功；
       * - 非0：失败。
       */
      virtual int SetRecordingSignalVolume(int volume) = 0;

      /**
       * @ingroup CPP_DingRtcEngineAudio
       * @since 3.5
       * @brief 获取音频采集音量。
       * @return 音量。
       */
      virtual int GetRecordingSignalVolume() = 0;

      /**
       * @ingroup CPP_DingRtcEngineVideo
       * @since 3.0
       * @brief 设置外部视频输入源。
       * @param enable 是否开启外部视频作为输入。
       * - true 开启。
       * - false 关闭。
       * @param track 视频流类型。
       * @return
       * - 0：成功；
       * - 非0：失败。
       */
      virtual int SetExternalVideoSource(bool enable, ding::rtc::RtcEngineVideoTrack track) = 0;

      /**
       * @ingroup CPP_DingRtcEngineVideo
       * @since 3.0
       * @brief 推送外部视频帧。
       * @param frame 外部视频帧，详见 {@link RtcEngineVideoFrame}。
       * @param track 视频流类型{@link RtcEngineVideoTrack}。
       * @return
       * - 0：成功；
       * - 非0：失败。
       */
      virtual int PushExternalVideoFrame(RtcEngineVideoFrame *frame, ding::rtc::RtcEngineVideoTrack track) = 0;
    
    /**
     * @ingroup CPP_DingRtcEngineVideo
     * @since 3.0
     * @brief 开始录制。
     * @details SDK提供了屏录录制的功能，可以录制本地或远端的音频流、视频流、相机流。
     * @param filePath 文件路径。
     * @param name 文件名称。
     * @return
     * - true ：成功；
     * - false ：失败。
     */
    virtual bool StartRecord(ding::rtc::RtcEngineRecordOptions recordOption,
                              const char * filePath,
                              const char * name) = 0;
    
    /**
     * @ingroup CPP_DingRtcEngineVideo
     * @since 3.0
     * @brief 更新录制内容信息。
     * @details SDK提供了更新录制内容信息的功能，用于实时更新录制模板内容设置。
     * @param layout 录制视频内容及布局， 详细定义见 {@link AliEngineRecordVideoLayout}。
     * @return
     * - true ：成功；
     * - false ：失败。
     * @note 此接口应该在录制过程中调用。
     */
    virtual bool UpdateRecordLayout(ding::rtc::RtcEngineRecordVideoLayout& layout) = 0;
    
    /**
     * @ingroup CPP_DingRtcEngineVideo
     * @since 3.0
     * @brief 暂停录制。
     * @details SDK提供了暂停录制的功能。
     * @return
     * - true ：成功；
     * - false ：失败。
     */
    virtual bool PauseRecord()  = 0;
    
    /**
     * @ingroup CPP_DingRtcEngineVideo
     * @since 3.0
     * @brief 重新开始录制。
     * @details SDK提供了重新开始录制的功能。
     * @return
     * - true ：成功；
     * - false ：失败。
     */
    virtual bool ResumeRecord()  = 0;
    
    /**
     * @ingroup CPP_DingRtcEngineVideo
     * @since 3.0
     * @brief 停止录制。
     * @details SDK提供了停止录制的功能。
     */
    virtual void StopRecord() = 0;
    
    /**
     * @ingroup CPP_DingRtcEngineVideo
     * @brief 对指定用户的视频进行截图，生成一张 JPG 格式的图片，并保存至指定的路径。
     * @param userId 用户ID，为 "" 代表本地用户。
     * @param trackType 视频流类型，只支持{@link RtcEngineVideoTrackCamera}和{@link RtcEngineVideoTrackScreen}。
     * @param filePath 截图的本地保存路径，需精确到文件名及格式，请确保目录存在且可写。如 xx/yy/zz.jpg 。
     * @return
     * - 0：仅代表调用该接口成功，是否获取到截图画面，详见{@link RtcEngineEventListener::OnSnapshotComplete}；
     * - < 0：失败。
     * @note 视频截图功能为异步接口，通过监听{@link RtcEngineEventListener::OnSnapshotComplete}回调获取当前截图画面。
     */
    virtual int SnapshotVideo(const char *userId, const RtcEngineVideoTrack &trackType, const char* filePath) = 0;

    /**
     * @ingroup CPP_DingRtcEngineMediaEngine
     * @brief 发送媒体扩展信息
     * @details SDK提供了发送和接收媒体扩展信息的功能，接收端参考 {@link RtcEngineEventListener::OnMediaExtensionMsgReceived}，使用场景：
     * - 使用媒体扩展信息传递时间戳，计算端到端的网络延迟，或者跟自身其他业务做数据同步
     * - 使用媒体扩展信息传递位控制信息。目前可以传递8 Byte数据，即64位，每一位或几位可以表示控制信息，用于自身业务上的指令传输
     *
     * @param message 扩展信息内容, 长度限制为最大8字节
     * @param length 扩展信息长度
     * @param repeatCount 重复次数，代表消息冗余度，用于防止网络丢包导致的消息丢失
     * @return
     * - 0： 成功
     * - <0： 失败
     *      - ERR_INNER(-1): SDK内部错误，可能的情况为SDK未初始化或者SDK销毁后调用
     *
     * @note 使用媒体扩展信息时需要复用音视频数据通道，因此必须控制自定义消息的发送频率和消息数据长度，使用限制如下：
     * - 每秒最多发送30条消息
     * - 为了不影响媒体数据的传输质量，自定义消息体长度限制为8 Byte，可以用来传输时间戳，位控制信息等
     * - sendMediaExtensionMsg函数中repeatCount参数为自定义消息冗余度，若大于1，则会发送多次，防止网络丢包导致的消息丢失，此时房间里的其他人也会收到多次相同的消息，需要去重
     * - 发送的自定义消息，在旁路直播时，房间里的订阅者也一样会收到
     * - 目前H5端不支持发送和接收媒体扩展信息
     */
    virtual int SendMediaExtensionMsg(unsigned char *message, unsigned int length, unsigned int repeatCount) = 0;

    /**
     * 关于音频分组概念：音频分组，允许频道内的成员建立工作组(group)。User加入一个group后，可以向这个group发言，
     * 也可以收听这个group内的音频。
     * 音频分组不影响视频的publish和subscribe，即使2个user不在同一个group，也可以看见对方。
     * 大厅(hall)：如果一个user当前不在任何一个group中，那么他就留在大厅中。大厅是一个特殊的
     * group，通过离开/加入一个group间接的达到加入/离开大厅的效果。
     * 如果user在大厅中，那么他可以对大厅中的人发言，也可以订阅大厅音频。
     * 权限约束：user无权对不在其中的group （含hall）发言，或者收听音频。
     * 音频订阅约束：user不可以同时订阅多个group （含hall）的音频，这个是为了应对一个user出现
     * 在多个group中，以及过量的多路音频（比如6路音频）混音后造成理解困难的问题。
     */

    /* App不能为大厅指定id。可以使用HallID判断一个group是不是大厅，还是普通的group
     */
    static const char * HallID;
    /**
     * @ingroup CPP_DingRtcEngineAudio
     * @since 3.4
     * @brief 加入音频分组。如果分组当前不存在，则自动创建一个。
     * @param groupId 指定加入的音频分组。不可以是大厅(HallID)，也不可以为空。允许的字符以及长度同频道ID。
     * @param usrData 可以额外提供一个字符串，业务层自行解释。长度不超过64字节。
     * @return int 如果返回0，结果将会通过{@link OnAudioGroupJoinResult}通知app。如果返回<0，则调用失败，不会有通知消息OnAudioGroupJoinResult。
     * @note usrData暂时未支持。
     */
    virtual int JoinGroup(const ding::rtc::String &groupId, const ding::rtc::String &usrData) = 0;
    /**
     * @ingroup CPP_DingRtcEngineAudio
     * @since 3.4
     * @brief 离开音频分组。
     * @param groupId 要离开的音频分组。不可以是大厅(HallID)，也不可以为空。
     * @return int 如果返回0，结果将会通过{@link OnAudioGroupLeaveResult}通知app。如果返回<0，则调用失败，不会有通知消息OnAudioGroupLeaveResult。
     * @note 如果离开了分组，那么没有权限再订阅该分组的音频，也没有权限向该分组mix音频流。
     */
    virtual int LeaveGroup(const ding::rtc::String &groupId) = 0;
    /**
     * @ingroup CPP_DingRtcEngineAudio
     * @since 3.4
     * @brief 解散音频分组，所有成员被迫离开该音频分组。
     * @param groupId 要解散的音频分组。不可以是大厅（HallID），也不可以为空。
     * @return int 如果返回0，结果将会通过{@link OnAudioGroupDismissResult}通知app。如果返回<0，则调用失败，不会有通知消息OnAudioGroupDismissResult。
     */
    virtual int DismissGroup(const ding::rtc::String &groupId) = 0;
    /**
     * @ingroup CPP_DingRtcEngineAudio
     * @since 3.4
     * @brief 本地音频合并进分组音频或者大厅音频中。
     * @param mix true表示合并，false表示不合并。
     * @param groupId 目标分组。如果是{@link HallID}，则表示目标为大厅的音频。不可以为空。
     * @return int 如果返回0，结果将会通过{@link OnAudioGroupMixResult}通知app。如果返回<0，则调用失败，不会有通知消息OnAudioGroupMixResult。
     * @note 前提条件：1，已经做了音频的publish; 2，权限要求：用户需要在目标分组或者大厅中。
     */
    virtual int MixAudioToGroup(bool mix, const ding::rtc::String &groupId) = 0;
    /**
     * @ingroup CPP_DingRtcEngineAudio
     * @since 3.4
     * @brief 切换订阅的音频流到指定的分组或者大厅音频。
     * @param groupId 指定的音频分组。如果是{@link HallID}，则表示目标为大厅的音频。如果groupId为空，则取消订阅当前的音频（无论是大厅还是某个分组的音频）。
     * @return int 如果返回0，结果将会通过{@link OnAudioSubscribeStateChanged}通知app。如果返回<0，则调用失败，不会有通知消息OnAudioSubscribeStateChanged。
     * @note 业务层监听OnAudioSubscribeStateChanged通知，如果uid为""，检查newState == RtcEngineStatsSubscribed，再查询当前订阅成功的音频类型（通过{@link GetCurrentSubscribedAudio}），可判断是否切换成功。
     * @note 切换过程会有多次OnAudioSubscribeStateChanged通知。
     * @note 权限约束：如果订阅指定的分组或者大厅音频，那么local user必须身在其中。如果取消订阅，那么不检查权限，总是可以安全的释放掉资源。
     */
    virtual int SwitchSubscriptionToGroup(const ding::rtc::String &groupId) = 0;
    /**
     * @ingroup CPP_DingRtcEngineAudio
     * @since 3.4
     * @brief 查询当前订阅的音频的类型，可能是大厅音频，也可能是某个分组的音频。
     * @return groupId 如果空，表示无订阅，如果是{@link HallID}，表示大厅音频，否则，返回分组音频对应的分组ID。
     */
    virtual ding::rtc::String GetCurrentSubscribedAudio() = 0;

    /**
     * @since 3.5
     * @brief 给音频分组设置名称。
     * @param groupId 指定的音频分组。如果是{@link HallID}，那么不成功，返回-1。
     * @param name 新的名称，替换掉旧的名称。如果传入空字符串，会移除旧的名称。
     * @return int 如果返回0，结果将会通过{@link OnSetGroupNameResult}通知app。如果返回<0，则调用失败，不会有通知消息OnSetGroupNameResult。
     * @note 不论是本地修改了分组名称，还是远端修改了分组名称，都有另一个消息{@link OnGroupNameChanged}通知app。
     */
    virtual int SetGroupName(const ding::rtc::String &groupId, const ding::rtc::String &name) = 0;
  /**
  * @}
  */
};

#ifdef __cplusplus
extern "C" {
#endif
    int RtcARGBToNV12(const unsigned char* src_argb, int src_stride_argb, unsigned char* dst_y, int dst_stride_y, unsigned char* dst_uv, int dst_stride_uv, int width, int height);

    int RtcI420ToNV21(const unsigned char* src_y, int src_stride_y, const unsigned char* src_u, int src_stride_u, const unsigned char* src_v, int src_stride_v, unsigned char* dst_y, int dst_stride_y, unsigned char* dst_vu, int dst_stride_vu, int width, int height);

    int RtcI420Rotate(const unsigned char* src_y, int src_stride_y, const unsigned char* src_u, int src_stride_u, const unsigned char* src_v, int src_stride_v, unsigned char* dst_y, int dst_stride_y, unsigned char* dst_u, int dst_stride_u, unsigned char* dst_v, int dst_stride_v, int width, int height, int mode);

    int RtcI420Copy(const unsigned char* src_y, int src_stride_y, const unsigned char* src_u, int src_stride_u, const unsigned char* src_v, int src_stride_v, unsigned char* dst_y, int dst_stride_y, unsigned char* dst_u, int dst_stride_u, unsigned char* dst_v, int dst_stride_v, int width, int height);
#ifdef __cplusplus
}
#endif
//! @}

} // namespace rtc
} // namespace ding

#endif /* __DING_RTC_ENGINE_INTERFACE_H__ */
