﻿#if !defined(__DING_RTC_ENGINE_WB_TYPES_H__)
#define __DING_RTC_ENGINE_WB_TYPES_H__

#include "engine_types.h"
#include "engine_wb_types.h"
#include <inttypes.h>
#include <cstddef>

namespace ding {
namespace rtc {

//! \addtogroup Basic_Datatypes  数据类型参考
//! @{

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 白板模式。
 */
enum class WBMode
{
    /** 基础模式。 */
    BASIC,
    /** 高级模式。 */
    ADVANCE
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 白板配置参数。
 */
struct WBConfig
{
    /** 白板模式。 */
    WBMode mode;
    /** 白板宽度。 */
    uint32_t width;
    /** 白板高度。 */
    uint32_t height;
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 标注类型。
 */
enum class AnnoType
{
    /** 共享标注 */
    SHARE,
    /** 视频标注 */
    VIDEO,
    /** 外部标注 */
    EXTERNAL
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 标注配置参数。
 */
struct AnnoConfig
{
    /** 标注类型。 */
    AnnoType type;
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 白板用户成员。
 * 
 */

struct WBUserMember {
    String uid;
};


/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 白板角色类型。
 */
enum class WBRoleType
{
    /** 白板管理员。 */
    ADMIN,
    /** 普通白板参与者。 */
    ATTENDEE,
    /** 只看白板参与者。 */
    VIEWER
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 白板工具参数。
 */
enum class WBToolType
{
    /** 空 */
    NONE = 0,
    /** 选中 */
    SELECT = 1,
    /** 点击 */
    CLICK = 2,
    /** 拖拽 */
    HAND = 3,
    /** 画笔 */
    PATH = 4,
    /** 直线 */
    LINE = 5,
    /** 矩形 */
    RECT = 6,
    /** 椭圆 */
    ELLIPSE = 7,
    /** 图片 */
    IMAGE = 8,
    /** 文本 */
    TEXT = 9,
    /** 删除工具 */
    DELETER = 10,
    /** 刷子 */
    BRUSH = 11,
    /** 箭头 */
    ARROW = 12,
    /** 折线。仅支持Windows及macOS平台。 */
    POLYLINE = 13,
    /** 弧线。仅支持Windows及macOS平台。 */
    ARC = 15,
    /** 曲线。仅支持Windows及macOS平台。 */
    CURVE = 16,
    /** 激光笔。 */
    LASER = 17,
    /** 图章 */
    STAMP = 18,
    /** 消失笔 */
    VANISHPEN = 19,
    /** 荧光笔 */
    HIGHLIGHTER = 20,
    /** 橡皮擦 */
    ERASER = 21
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 白板填充类型。
 */
enum class WBFillType
{
    /** 不填充 */
    NONE,
    /** 填充颜色 */
    COLOR,
    /** 边框并填充 */
    BORDER_FILL,
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 点。 */
struct WBPoint
{
    /** x坐标 */
    float x{ 0.f };
    /** y坐标 */
    float y{ 0.f };
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 大小。 */
struct WBSize
{
    /** 宽度 */
    float cx{ 0.f };
    /** 高度 */
    float cy{ 0.f };
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 矩形。 */
struct WBRect
{
    /** 左上角x坐标 */
    float left{ 0.f };
    /** 左上角y坐标 */
    float top{ 0.f };
    /** 右下角x坐标 */
    float right{ 0.f };
    /** 右下角y坐标 */
    float bottom{ 0.f };
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 白板颜色。 */
struct WBColor
{
    /** 红色成分，有效值范围：[0.0, 1.0]。默认值：0.0。 */
    float r{ 0.f };
    /** 绿色成分，有效值范围：[0.0, 1.0]。默认值：0.0。 */
    float g{ 0.f };
    /** 蓝色成分，有效值范围：[0.0, 1.0]。默认值：0.0。 */
    float b{ 0.f };
    /** 透明度成分，有效值范围：[0.0, 1.0]。默认值：1.0。 */
    float a{ 1.f };
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 白板字体样式。 */
enum class WBFontStyle{
    /** 常规 */
    NORMAL,
    /** 粗体 */
    BOLD,
    /** 斜体 */
    ITALIC,
    /** 粗斜体 */
    BOLD_ITALIC
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 白板文本格式。 */
struct WBTextFormat{
    /** 字体大小，有效值范围：[10, 96]。默认值：12。 */
    uint32_t size{ 12 };
    /** 字体样式 */
    WBFontStyle style{ WBFontStyle::NORMAL };
    /** 字体颜色 */
    WBColor color{};
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 白板图章。 */
struct WBStamp
{
    /** 图章资源ID。最大长度128字节 */
    const char *stampId = nullptr;
    /** 图章资源路径 */
    const char *path = nullptr;
    /** 图章是否可以改变大小。默认值：false。*/
    bool resizable = false;
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 白板图像状态。 */
enum class WBImageState
{
    /** 开始加载图片。 */
    LOAD_START,
    /** 图片加载成功。 */
    LOAD_COMPLETE,
    /** 图片加载失败。 */
    LOAD_FAIL
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 文件转码状态。 */
enum class WBFileTransState
{
    /** 开启转码。 */
    TRANSCODE_START,
    /** 转码成功。 */
    TRANSCODE_COMPLETE,
    /** 转码失败。 */
    TRANSCODE_FAIL,
    /** 转码开启失败 */
    TRANSCODE_START_FAIL,
    /** 转码查询失败 */
    TRANSCODE_QUERY_FAIL
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 白板渲染缩放模式。 */
enum class WBScalingMode
{
    /** 保持宽高比 */
    Fit,
    /** 保持宽高比全填充 */
    CropFill,
    /** 原始大小居中显示 */
    CenterCrop,
    /** 拉伸铺满全填充 */
    StretchFill
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 白板渲染缩放基准。 */
enum class WBScalingBase
{
    /** 默认。100%对应原始白板大小。 */
    DEFAULT = 0,
    /** 视图基准。100%对应FIT填充视图。 */
    VIEW
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 白板清除模式。 */
enum class WBClearMode
{
    /** 全部内容。 */
    ALL,
    /** 其它人内容。 */
    OTHERS,
    /** 自己内容。 */
    SELF,
    /** 指定内容。 */
    SPECIFIC
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 白板清除配置。 */
struct WBClearParam
{
    /** 是否仅清除当前页。 */
    bool curPage{ true };
    /** 白板清除模式 */
    WBClearMode mode;
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 绘制事件类型 */
enum class WBDrawEvent
{
    /** 未知事件。 */
    DRAW_UNKNOWN,
    /** 绘制开始。 */
    DRAW_START,
    /** 绘制结束。 */
    DRAW_END,
    /** 绘制取消。 */
    DRAW_CANCEL,
    /** 选择移动开始。 */
    SELECT_MOVE_START,
    /** 选择移动结束。 */
    SELECT_MOVE_END,
    /** 删除开始。 */
    DELETE_START,
    /** 删除结束。 */
    DELETE_END,
    /** 文本绘制。 */
    TEXT_DRAW,
    /** 文本更新。 */
    TEXT_UPDATE,
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 内容更新类型。 */
enum class WBContentUpdateType
{
    /** 未知。 */
    UNKNOWN,
    /** 远端绘制。 */
    REMOTE_DRAW
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 快照模式 */
enum class WBSnapshotMode
{
    /** 视图显示区域。 */
    VIEW = 0,
    /** 全部区域。 */
    ALL,
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 白板页码定义。 */
using WBPageNumber = uint32_t;
/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 白板非法页码。 */
const WBPageNumber kInvalidPageNumber = WBPageNumber(-1);

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 白板文档类型。 */
enum class WBDocType
{
    /** 普通。 */
    NORMAL = 1,
    /** PDF。 */
    PDF = 3,
    /** 外部。 */
    EXTERNAL = 5,
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 白板文档信息。 */
struct WBDocInfo
{
    /** 文档类型。 */
    WBDocType type;
    /** 文档ID。 */
    String docId{ nullptr };
    /** 文档名称。 */
    String name{ nullptr };
    /** 文档创建人用户ID。 */
    String creator{ nullptr };
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 白板文档内容信息 */
struct WBDocContents
{
    /** 文档类型 */
    WBDocType type{ WBDocType::NORMAL };
    /** 文档名称 */
    String name;
    /** url地址数组 */
    StringArray urls;
    /** 文档转码ID */
    String transDocId;
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 白板外部文档信息 */
struct WBDocExtContents
{
    /** 外部文档名称 */
    String name;
    /** 总页数 */
    size_t totalPages{1};
    /** 文档宽度 */
    uint32_t width;
    /** 文档高度 */
    uint32_t height;
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 白板触摸配置。 */
struct WBTouchConfig
{
    /** 是否开启手掌检测。 */
    bool enable{ false };
    /** 小接触大小。 */
    float smallContactSize{ 0.f };
    /** 大接触大小。 */
    float largeContactSize{ 0.f };
    /** 小橡皮擦大小。 */
    float smallEraserSize { 100.f };
    /** 大橡皮擦大小。 */
    float largeEraserSize { 200.f };
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 白板选项。 */
enum class WBOption
{
    /** 启用白板响应UI事件。默认启用 */
    ENABLE_UI_RESPONSE  = 2, // type: bool
    /** 显示或隐藏白板涂鸦。默认显示
     * @note 背景图和H5内容不受此选项影响 */
    ENABLE_SHOW_DRAWS  = 3, // type: bool
    /** 启用白板画布缩放移动。默认启用
     * @note SetZoomScale 不受此选项的影响 */
    ENABLE_SCALE_MOVE  = 4, // type: bool
    /** 设置图片或音视频对象默认选中状态。默认启用 */
    ENABLE_AUTO_SELECTED = 5, // type: bool
    /** 启用光标位置同步。默认关闭 */
    ENABLE_CURSORPOS_SYNC = 6, // type: bool
    /** 开启显示远端光标。默认关闭 */
    ENABLE_SHOW_REMOTE_CURSOR = 7, // type: bool
    /** 是否显示本地用户名称。默认关闭 */
    ENABLE_LOCAL_CURSOR_LABEL = 9, // type: bool
    /** 选中时是否显示对象用户名称。默认打开 */
    ENABLE_SELECT_SHOW_NAME = 10, // type: bool
    /** 打开触屏模式。默认关闭 */
    ENABLE_TOUCH_SCREEN = 11, // type: bool
    /** 打开激光笔尾巴。默认打开 */
    ENABLE_LASER_TRAIL = 12, // type: bool
    /** 打开橡皮擦轨迹。默认关闭 */
    ENABLE_ERASE_TRAIL = 13, // type: bool
    /** 开启渲染线程。默认关闭。当前仅对Windows有效 */
    ENABLE_RENDER_THREAD = 14, // type: bool
    /** 设置选择和删除热区大小。默认值10.f (float类型) */
    HOT_ZONE_SIZE = 15, // type: float
    /** 设置触摸点光标大小。默认值32 (uint32_t类型) */
    TOUCH_CURSOR_SIZE = 16, // type: uint32_t
    /** 开启手掌橡皮擦。默认关闭 */
    ENABLE_PALM_ERASER = 17, // type: bool
    /** 设置橡皮擦热区大小。默认值100.f (float类型) */
    ERASER_ZONE_SIZE = 18,
    /** 触摸配置。默认关闭 */
    TOUCH_CONFIG = 19, // type: WBTouchConfig
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.5
 * @brief 标注选项。 */
enum class AnnoOption
{
    /** 启用本地标注渲染。默认启用。
     * @note 该选项需要在调用open前设置 */
    ENABLE_LOCAL_RENDER         = 1, // type: bool
    /** 显示或隐藏标注涂鸦。默认显示 */
    ENABLE_SHOW_DRAWS           = 2, // type: bool
    /** 启用标注响应UI事件。默认启用 */
    ENABLE_UI_RESPONSE          = 3, // type: bool
    /** 启用标注画布缩放移动。默认启用
     * @note SetZoomScale 不受此选项的影响 */
    ENABLE_SCALE_MOVE           = 4, // type: bool
    /** 启用光标位置同步。默认关闭 */
    ENABLE_CURSORPOS_SYNC       = 5, // type: bool
    /** 开启显示远端光标。默认关闭 */
    ENABLE_SHOW_REMOTE_CURSOR   = 6, // type: bool
    /** 缩放基准模式。默认WBScalingBase::Default */
    SCALE_BASE_MODE = 8, // type: WBScalingBase
    /** 是否显示本地用户名称。默认关闭 */
    ENABLE_LOCAL_CURSOR_LABEL = 9, // type: bool
    /** 是否自动跟踪窗口大小和位置。默认开启，如关闭需依赖setWindowPos设置。 */
    ENABLE_AUTO_TRACKING_WINDOW = 10, // type: bool
    /** 选中时是否显示对象用户名称。默认打开 */
    ENABLE_SELECT_SHOW_NAME = 11, // type: bool
    /** 打开触屏模式。默认关闭 */
    ENABLE_TOUCH_SCREEN = 12, // type: bool
    /** 打开激光笔尾巴。默认打开 */
    ENABLE_LASER_TRAIL = 13, // type: bool
    /** 打开橡皮擦轨迹。默认关闭 */
    ENABLE_ERASE_TRAIL = 14, // type: bool
    /** 本地绘制开启低刷新率。默认关闭
     * @note 仅对低渲染性能设备设置 */
    ENABLE_LOCAL_LOW_REFRESH_RATE = 15, // type: bool
    /** 开启渲染线程。默认关闭
     * @note 当前仅对Windows有效 */
    ENABLE_RENDER_THREAD = 16, // type: bool
    /** 设置选择和删除热区大小。默认值10.f (float类型) */
    HOT_ZONE_SIZE = 17, // type: float
    /** 设置触摸点光标大小。默认值32 (uint32_t类型) */
    TOUCH_CURSOR_SIZE = 18, // type: uint32_t
    /** 设置橡皮擦热区大小。默认值100.f (float类型) */
    ERASER_ZONE_SIZE = 19, // type: float
};

//! @}

} // namespace rtc
} // namespace ding

#endif /* __DING_RTC_ENGINE_WB_TYPES_H__ */
